//
// USAFreightandPassengerRun.gs
//
//  Copyright (C) 2002 Auran Developments Pty Ltd
//  All Rights Reserved.
//

include "trainz.gs"
include "world1.gs"
include "common.gs"
include "browser.gs"
include "stringtable.gs"


//
// class MyUSAFreightPass
// brief This is the scenario class.  Modify this class with
// your own gameplay.
//
game class MyUSAFreightPass isclass Scenario, ScenarioSupport
{
  bool scenarioDone = false;
  bool recentBadCouple = false;
  string objectiveIcon = null;

  Train userTrain, aiFreight, passengerCars, droppedOff;
  Schedule passingFreightSchedule;
  StationBehaviour stationBehaviour;
  Signal tunnelSignal, bridgeSignal, yardSignal;

  StringTable stringTable; // scenario string table


  //
  // Load will be called by Trainz to load the scenario map, and when the user presses Ctrl-L
  // param data is the save game data if loading a saved game.
  //
  bool Load(string data)
  {
    if(data and data.size())
    {
      Interface.Load(data);
    }

    // initialize scenario string table
    stringTable = ScenarioStringTable.Get();

    // load the map
    if(!World.LoadMap(World.FindKUID("northamerica")))
    {
      Interface.Log(stringTable.GetString1("std_error_loading_map_name", "northamerica"));
      return false;
    }

    // disable saving on this scenario
    BrowserInterface.SetSaveAllowed(false);


    //
    // Create consists
    //

    // user train
    userTrain = World1.CreateTrain("userfreight", "START HERE 2", true);
//    userTrain = World1.CreateTrain("alcos", "StartY", true); // test/debug
    userTrain.SetFloatingLimit(10.0 * Train.KPH_TO_MPS, true);

    // AI freight that goes out of the mine
    aiFreight = World1.CreateTrain("LoadedCoalTrain", "StartAIFreight", true);

    // passenger cars at central
    passengerCars = World1.CreateTrain("passpickup", "PLATFORM1", true);
    passengerCars.Turnaround();


    //
    // Schedules
    //
    
    // AI freight schedule
    passingFreightSchedule = new Schedule();
    passingFreightSchedule.TakeJunction("BLUE SKY MINE", Junction.DIRECTION_BACKWARD);
    passingFreightSchedule.TakeJunction("BLUE SKY STH", Junction.DIRECTION_BACKWARD);
    passingFreightSchedule.TakeJunction("Steel_Mills", Junction.DIRECTION_RIGHT);
    passingFreightSchedule.TakeJunction("CENTRAL STATION", Junction.DIRECTION_LEFT);
    passingFreightSchedule.TakeJunction("MAIN YARD WEST", Junction.DIRECTION_LEFT);
    passingFreightSchedule.StopTrain(10);


    //
    // Initialize station behaviour
    //
    stationBehaviour = new StationBehaviour();
    stationBehaviour.SetDefaults();
    stationBehaviour.warning.objective = "";
    stationBehaviour.warning.icon = "pause.tga";
    stationBehaviour.overrun.icon = "bad.tga";
    stationBehaviour.reverse.icon = "bad.tga";
    stationBehaviour.stopped.objective = "";
    stationBehaviour.mwstopped.icon = "bad.tga";
    stationBehaviour.mwstopped.alert = stringTable.GetString("std_bad_guard_clearence");
    stationBehaviour.mwstopped.colour = Interface.Colour_Red;
    stationBehaviour.stopped.icon = "pause.tga";
    stationBehaviour.stopped.wav = "bell.wav";
    stationBehaviour.delay = 15;


    //
    // Junctions
    //

    // Steel Mill
    Navigate.LockJunction("MINERALS YARD", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("HIGH LINE", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("Steel_Mills", Junction.DIRECTION_LEFT, false);

    // Blue Sky Station
    Navigate.LockJunction("BLUE SKY STH", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("BLUE SKY NORTH", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("BLUE SKY MINE", Junction.DIRECTION_LEFT, false);

    // Marshalling yard
    Navigate.LockJunction("HARRISON TERMINAL", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("HARRISON ROUTE NTH", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("MARSHALLING YARD NTH", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("MY04", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("MY01", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("MARSHALLING YARD", Junction.DIRECTION_LEFT, false);
    
    // Central station
    Navigate.LockJunction("MAIN YARD", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("RIVER BRIDGE", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("HARRISON ROUTE", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("MAIN YARD STH", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("CENTRAL 04", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("CENTRAL 03", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("CENTRAL", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("PLATFORM 1", Junction.DIRECTION_LEFT, false);


    //
    // Signals
    //
    tunnelSignal = cast<Signal> Router.GetGameObject("TS");
    bridgeSignal = cast<Signal> Router.GetGameObject("B1");
    yardSignal   = cast<Signal> Router.GetGameObject("YS");

    tunnelSignal.SetSignalState(Signal.YELLOW, "Caution!");
    bridgeSignal.SetSignalState(Signal.YELLOW, "Caution!");
    yardSignal.SetSignalState(Signal.RED, "Stop here!");

    return true;
  }


  //
  // Save will be called by Trainz when the user presses Ctrl-S.
  // return the save game string, such that load will be able to restor the save game
  // from the last save check point.
  //  
  string Save()
  {
    return Interface.Save();
  }


  //
  // TrainDerailed() will be called by Trainz when a train derails
  //
  void TrainDerailed(int trainId) 
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_derailed_train"), "bad.tga");
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);
      scenarioDone = true;
    }
  }


  //
  // TrainCollided() will be called by Trainz when a train collides
  //
  void TrainCollided(int trainId) 
  {
    if(!scenarioDone)
    {
      Interface.SetObjective(stringTable.GetString("std_terminated_train_collision"), "bad.tga");       
      World.Play2DSound("ALARM.wav");
      World.EndScenario(5);  
      scenarioDone = true;
    }
  }


  //
  // TrainSpeedingFine() is called by Trainz every second your trains speed exceeds the floating limit
  //
  void TrainSpeedingFine() 
  {
    Interface.AdjustScore(-1);  
  }


  //
  // Warns user of their bad couple-up and displays a message along with alarm sound.
  //
  thread void BadCouple()
  {
    // set to true so this method won't be called repeatedly in near proximity
    recentBadCouple = true;

    // warn user and deduct points
    Interface.SetObjective(stringTable.GetString("std_bad_couple_imperial"), "bad.tga");
    Interface.AdjustScore(-50);
    World.Play2DSound("ALARM.wav");

    // recentDecouple reset after delay so we don't get 2 BadCouple() calls too close to each other
    Sleep(1.0);
    recentBadCouple = false;

    // after brief sleep, hide warning message and restore previous icon
    Sleep(3.0);
    Interface.SetObjective("", objectiveIcon);
    Interface.ShowObjective(false);
  }


  //
  // TrainBadCouple() is called by Trainz when vehicles couple greater than 8kph (5mph).
  //
  void TrainBadCouple(int vehicleId)
  {
    // was it a recent bad couple (last 3 seconds)? If so, display message and alarm
    if (!recentBadCouple) 
      BadCouple();
  }



  //
  // main thread
  // brief main is executed automatically after Load() is called.  edit
  // main to contain your scenarios gameplay.
  //
  thread void main(void)
  {
    // Start the monitor thread to monitor speeding, derailing etc.
    Monitor();

    // Let user select DCC or Cabin Control for the scenario
    SelectCabinDCC(userTrain);

    // Start off with 1000 points.  This will give us some points to play with before
    // the user obtains a negative score (if they do things wrong)
    Interface.AdjustScore(1000);

    // switch light of user's loco on
    SendMessage(userTrain, "loco", "light"); 


    //
    // Train starts out at mill...
    //

    // display instruction sheet and setup thread to trap keystroke so it can be called back again
    DoInstructions();
    ManageBrowser();

    // start off..
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");
    Sleep(Math.Rand(2, 5));
    World.Play2DSound("lines_clear.wav");

    // start off the AI train that passes through
    Navigate.OnTrigger(me, userTrain, "StartAICoal", Navigate.TRIGGER_ENTER);
    aiFreight.RunSchedule(passingFreightSchedule, false, 0);
    SendMessage(aiFreight, "loco", "light");  // switch light of loco on

    // once user train is clear of single track, change junctions so AI train can get through and blast it's horn
    Navigate.OnTrigger(me, userTrain, "WarnBlueSky", Navigate.TRIGGER_ENTER);
    SendMessage(aiFreight, "loco", "horn");
    Navigate.LockJunction("BLUE SKY STH", Junction.DIRECTION_LEFT, false);

    // 200 points for passing through BlueSky
    Interface.AdjustScore(200);
    Interface.Print(stringTable.GetString1("alcop_objective_coalpass", "200"));
    Interface.Log(stringTable.GetString1("alcop_objective_coalpass", "200"));

//    Navigate.OnJunction(me, userTrain, "HARRISON TERMINAL", Navigate.JUNCTION_ENTER); // checkpoint


    //
    // Do drop off in marshalling yard
    //

    // warn user to slow down with pause icon as they enter the siding
    Navigate.OnTrigger(me, userTrain, "TrigWarnYard", Navigate.TRIGGER_ENTER);
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");

    // warn user to stop at end of siding
    Navigate.OnTrigger(me, userTrain, "Trig FreightDrop", Navigate.TRIGGER_ENTER);
    objectiveIcon = "stop.tga";
    Interface.SetObjectiveIcon("stop.tga");

    // Wait for the user to do decouple operation
    Navigate.OnTrigger(me, userTrain, "Trig FreightDrop", Navigate.TRIGGER_STOPPED);
    objectiveIcon = "decouple.tga";
    Interface.SetObjectiveIcon("decouple.tga");
    droppedOff = TrainUtil.DoDecouple(userTrain, 15);
    Interface.AdjustScore(300);

    // user has dropped off stuff in yard, they can now proceded onto the staion
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");


    //
    // Do pickup at Central
    //

    // warn user to slow down as the get near platform 1
    Navigate.OnTrigger(me, userTrain, "TrigWarnCentral", Navigate.TRIGGER_ENTER);
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");

    // wait for user to enter siding and display coupler icon
    Navigate.OnTrigger(me, userTrain, "TrigWarnCouple", Navigate.TRIGGER_ENTER);
    objectiveIcon = "decouple.tga";
    Interface.SetObjectiveIcon("decouple.tga");

    // couple up with passenger cars
    TrainUtil.DoCoupleUp(userTrain, passengerCars);
    Interface.AdjustScore(300);
    Sleep(4);

    // reverse logical direction of coupled up passenger train
    userTrain.Turnaround();
    World.SetCamera(userTrain.GetFrontmostLocomotive(), World.CAMERA_INTERNAL);

    // await guard's clearence
    objectiveIcon = "pause.tga";
    Interface.SetObjectiveIcon("pause.tga");
    Sleep(Math.Rand(3, 5) + World.Play2DSound("all_aboard.wav"));
    Sleep(Math.Rand(25, 30));
    Sleep(World.Play2DSound("Buzzer.wav"));

    // set junctions so train can get to Blue SKY
    Navigate.LockJunction("CENTRAL", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("CENTRAL NTH", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("Jefferson", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("HARRISON TERMINAL", Junction.DIRECTION_LEFT, false);
    Navigate.LockJunction("BLUE SKY NORTH", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("BLUE SKY MINE", Junction.DIRECTION_RIGHT, false);
    Navigate.LockJunction("BLUE SKY STH", Junction.DIRECTION_LEFT, false);

    // Inform user to depart
    Sleep(Math.Rand(3, 5));
    objectiveIcon = "play.tga";
    Interface.SetObjectiveIcon("play.tga");

    // 100 points for coupling up with coaches at Central
    Interface.AdjustScore(100);
    Interface.Print(stringTable.GetString1("alcop_objective_startcentral", "100"));
    Interface.Log(stringTable.GetString1("alcop_objective_startcentral", "100"));


    //
    // Take train to BlueSky
    //
    Navigate.Station(me, userTrain, "Blue Sky", "WarnBlueSky", "StopBlueSky", "OverBlueSky", "", stationBehaviour, 20, 30);

    // 300 points for Blue Sky stop
    Interface.AdjustScore(300);
    Interface.Print(stringTable.GetString1("alcop_objective_bluesky", "300"));
    Interface.Log(stringTable.GetString1("alcop_objective_bluesky", "300"));
    Sleep(Math.Rand(3, 5));


    // finish off the scenario
    Interface.SetObjective(stringTable.GetString1("scenario_complete_objective", Interface.GetScore()), "tick.tga");
    Interface.SetResults();
    Interface.LogResult(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    Interface.Print(stringTable.GetString1("scenario_complete", Interface.GetScore()));
    scenarioDone = true;
    World.EndScenario(8.0);
  }
};